/*
 * 	 Cholesky.java
 * 
 * 
 *   famCNV: Copy Number Variant association for quantitative traits in families
 *   Copyright (C) 2010 Dr. Mario Falchi                                   

 *
 *   famCNV is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   famCNV is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with famCNV.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   For any bugs or problems found, please contact us at
 *   m.falchi@imperial.ac.uk  
 *	 hariklia.eleftherohorinou06@imperial.ac.uk
 *   visconti@di.unito.it
 */



class Cholesky
{
    private double[][] L;
    private double[] x;
    private int dim;


    public Cholesky()
    {
        dim = 0;
    }
    
    public double[] x()
    {
        return x;
    }
    
    public void fastDecompose(double[][] A) throws Exception
    {
        if (A[0].length != A.length) 
            throw new Exception("Cholesky: the matrix is not square");
        
        dim = A.length;
        L = new double[dim][dim];
        
        for (int i=0; i<dim; i++) 
        {
            for (int j=i; j<dim; j++) 
            {
                double sum = A[i][j];
                for (int k = i-1; k >= 0; k--)
                    sum -= L[i][k] * L[j][k];
                    
                if (i==j) 
                {
                    if (sum <= 0.0)
                      throw new Exception("Cholesky: the matrix is not positive definite");
                    else
                        L[i][i] = Math.sqrt(sum);
                } 
                else
                    L[j][i] = sum / L[i][i];
            }
        }
    }

    public void backSubst(double[] b) 
    {
        x = Utilities.set(dim, 0.0);
        
        //solve L*x = b 
        for (int i = 0; i<dim; i++) 
        {
            double sum = b[i];
            for (int k = i-1; k >= 0; k--)
                sum -= L[i][k] * x[k];
            x[i] = sum / L[i][i];
        }
        
        //solve transpose(L)*x = v
        //end result is A*x = L*t(L)*x = L*v = b)
        for (int i = dim-1; i >= 0; i--) 
        {
            double sum = x[i];
            for (int k = i+1; k < dim; k++)
                sum -= L[k][i] * x[k];
            x[i] = sum / L[i][i];
        }
    }

    public double lnDeterminantL() 
    {
        double sum = 0.0;
        for (int i = 0; i < dim; i++)
            sum += Math.log(L[i][i]);
        
        return sum;
    }
}
